import logging
from logging.handlers import RotatingFileHandler

logging.root.setLevel(logging.DEBUG)
test_pioneer_logger = logging.getLogger("TestPioneer")
formatter = logging.Formatter('%(asctime)s | %(name)s | %(levelname)s | %(message)s')


class TestPioneerHandler(RotatingFileHandler):

    def __init__(self, filename: str = "TestPioneer.log", mode="w",
                 maxBytes: int = 1073741824, backupCount: int = 0):
        super().__init__(filename=filename, mode=mode, maxBytes=maxBytes, backupCount=backupCount)
        self.formatter = formatter
        self.setLevel(logging.DEBUG)

    def emit(self, record: logging.LogRecord) -> None:
        super().emit(record)


def step_log_check(enable_logging: bool = False, logger: logging.Logger = None,
                   level: str = "info", message: str = None) -> None:
    if enable_logging and logger:
        logger_level = {
            "info": logger.info,
            "error": logger.error,
        }.get(level, None)
        if logger_level is not None:
            logger_level(message)



from test_pioneer.logging.loggin_instance import TestPioneerHandler, test_pioneer_logger


def set_logger(yaml_data: dict) -> bool:
    if "pioneer_log" in yaml_data.keys():
        enable_logging = True
        filename = yaml_data.get("pioneer_log")
        file_handler = TestPioneerHandler(filename=filename)
        test_pioneer_logger.addHandler(file_handler)
        return True
    return False


from test_pioneer.logging.loggin_instance import test_pioneer_logger, step_log_check


def download_single_file(step: dict, enable_logging: bool = False) -> bool:
    file_url = step.get("download_file")
    file_path = step.get("file_path")
    from automation_file import download_file
    if file_url is None or file_path is None:
        step_log_check(
            enable_logging=enable_logging, logger=test_pioneer_logger, level="info",
            message=f"Please provide the file_url and download_file")
        return False
    if isinstance(file_url, str) is False or isinstance(file_path, str) is False:
        step_log_check(
            enable_logging=enable_logging, logger=test_pioneer_logger, level="info",
            message=f"Both file_url and download need to be of type str")
        return False
    download_file(file_url=file_url, file_name=file_path)
    return True


def unzip_zipfile(step: dict, enable_logging: bool = False) -> bool:
    zip_file_path = step.get("zip_file_path")
    password = step.get("password")
    extract_path = step.get("extract_path")
    if zip_file_path is None:
        step_log_check(
            enable_logging=enable_logging, logger=test_pioneer_logger, level="info",
            message=f"Please provide the zip_file_path")
        return False
    if not isinstance(zip_file_path, str):
        step_log_check(
            enable_logging=enable_logging, logger=test_pioneer_logger, level="info",
            message=f"zip_file_path need to be of type str")
        return False

    from automation_file import unzip_all
    kwargs = {
        "zip_file_path": zip_file_path,
    }
    if password:
        kwargs["password"] = password
    if extract_path:
        kwargs["extract_path"] = extract_path

    unzip_all(**kwargs)

    return True



import os
from pathlib import Path

from test_pioneer.executor.run.utils import select_with_runner
from test_pioneer.logging.loggin_instance import step_log_check, test_pioneer_logger


def run_folder(step: dict, enable_logging: bool = False, mode: str = "run_folder"):
    check_with_data = select_with_runner(step, enable_logging=enable_logging, mode=mode)
    if check_with_data[0] is False:
        return False
    else:
        execute_with = check_with_data[1]
    folder = step.get("run_folder")
    folder = str(Path(os.getcwd() + folder).absolute())
    if folder is None:
        step_log_check(
            enable_logging=enable_logging, logger=test_pioneer_logger, level="error",
            message=f"run param need folder path: {step.get('run_folder')}")
        return False
    if (Path(folder).is_dir() is False) or not Path(folder).exists():
        step_log_check(
            enable_logging=enable_logging, logger=test_pioneer_logger, level="error",
            message=f"This folder not exists: {step.get('run_folder')}")
        return False
    folder = Path(folder)
    json_files = list(folder.glob('*.json'))
    if len(json_files) > 0:
        execute_with(json_files)
    else:
        step_log_check(
            enable_logging=enable_logging, logger=test_pioneer_logger, level="error",
            message=f"Folder is empty: {step.get('run_folder')}")
        return False
    return True